#include <stdio.h>
#include <regex.h>

/* Highlight a pattern in the input by coloring it in the output. */
/* "regex" is used to find the patterns.                          */

  static char Id[] = "$Id: hl.c,v 1.6 2004/10/30 00:32:46 vsnyder Exp $";

main ( int argc, char* argv[] )
{ char b[256];                      /* input buffer */
  char after[] = "\033[0m";         /* black on transparent background */
  char blue[] = "\033[00;34;1m";    /* bold blue on transparent background */
  char magenta[] = "\033[00;35;1m"; /* bold magenta on transparent background */
  char red[] = "\033[00;31;1m";     /* bold red on transparent background */
  int i;                            /* Subscript/loop inductor */

  typedef struct
  { char* find;          /* Pattern to find */
    char* color;         /* String to but before it */
    regex_t preg;        /* Compiled pattern -- from regcomp */
  } pat;

  regmatch_t match;      /* Where is the string that matches the pattern? */

  pat pats[] = { "[0-9]+-[SU]", red, {},     /* lf95 errors */
                 "[0-9]+-W", blue, {},       /* lf95 warnings */
                 "[0-9]+-I", magenta, {},    /* lf95 informative */
                 /* NAG f95 patterns: */
                 "Deleted", red, {},       "Error", red, {},
                 "Fatal", red, {},         "Panic", red, {},
                 "Warning", blue, {},      "Extension", blue, {},
                 "Obsolescent", blue, {},  "Info", magenta, {},
                 "undefined", red, {} };     /* during linking */

#define NPAT sizeof pats / sizeof pats[0]

  /* Compile the patterns */
  for ( i=0; i<NPAT ; i++ ) regcomp ( &pats[i].preg, pats[i].find, REG_EXTENDED );

  setvbuf ( stdout, (char*)NULL, _IONBF, 1 ); /* unbuffer the output */

  while (1)
  { /* Get a line: */
    if ( fgets ( b, sizeof b, stdin ) == NULL ) return(0);
    /* Look for a pattern match */
    for ( i=0; i<NPAT; i++ )
    { if ( regexec ( &pats[i].preg, b, 1, &match, 0 ) == 0 )
      /* Got a match; put the desired color around it */
      { fwrite ( b, sizeof(char), match.rm_so, stdout );
        printf ( "%s", pats[i].color );
        fwrite ( &b[match.rm_so], sizeof(char), match.rm_eo - match.rm_so, stdout );
        printf ( "%s%s", after, &b[match.rm_eo] );
        goto cycle;
      }
    }
    /* No match; just echo the input */
    printf ( "%s", b );
  cycle:
  }
}

/*
$Log: hl.c,v $
Revision 1.6  2004/10/30 00:32:46  vsnyder
Changed 'puce' to 'magenta'

Revision 1.5  2004/10/06 23:44:46  vsnyder
Add 'colors' bash script at the end as a comment

Revision 1.4  2001/03/03 02:15:48  vsnyder
Correct misunderstanding about usage of regex

Revision 1.3  2001/03/03 02:04:24  vsnyder
Committing working version

*/

/*
#!/bin/bash
# Display ANSI colours.
#
esc="\033["
echo -n "FG/ BG: _40_ _ _ 41 _ _ _42_ _ _ 43"
echo " _ _ 44 _ _ _45_ _ _ 46 _ _ _47_ _ _ 00_"
for fore in 30 31 32 33 34 35 36 37 00; do
  line1="$fore  "
  line2="    "
  for back in 40 41 42 43 44 45 46 47 00; do
    line1="${line1}${esc}${back};${fore}m Normal ${esc}0m"
    line2="${line2}${esc}${back};${fore};1m Bold   ${esc}0m"
  done
  echo -e "$line1\n$line2"
done
*/
